package com.flyfish.framework.logging.advice;

import com.flyfish.framework.logging.service.LogContext;
import com.flyfish.framework.logging.service.LogManager;
import lombok.RequiredArgsConstructor;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;

import javax.annotation.Resource;

/**
 * 日志切面
 *
 * @author wangyu
 */
@Aspect
@RequiredArgsConstructor
public class LogAdvice {

    private final LogManager logManager;

    /**
     * 切入注解
     */
    @Pointcut("execution(public * com.flyfish..*.*Controller.*(..))")
    public void pointCut() {

    }


    /**
     * 环绕切面，获取参数
     *
     * @param joinPoint 切入点
     * @return 结果
     */
    @Around("pointCut()")
    public Object process(ProceedingJoinPoint joinPoint) throws Throwable {
        // 构建上下文
        LogContext context = LogContext.of(joinPoint);
        try {
            Object result = joinPoint.proceed(joinPoint.getArgs());
            context.setResult(result);
            return result;
        } catch (Throwable throwable) {
            context.setError(throwable);
            throw throwable;
        } finally {
            if (context.isValid()) {
                context.end();
                logManager.tryLog(context);
            }
        }
    }
}
