package com.flyfish.framework.beans.meta;

import com.flyfish.framework.domain.base.Po;
import lombok.Getter;

import java.beans.PropertyDescriptor;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.function.BiPredicate;

/**
 * bean属性的类型
 * 包含java类型到js的映射
 * 我们通过类型自动解析bean的缺省控件，并自动映射
 * @author wangyu
 */
@Getter
public enum BeanPropertyType {

    STRING("String", CharSequence.class),
    NUMBER("Number", Number.class),
    BOOLEAN("Boolean", Boolean.class),
    DATE("Date", Date.class),
    ENUM("Enum", Enum.class),
    LIST("Array", Collection.class),
    DB_REF("Ref", Po.class),
    OBJECT("Object", Object.class);

    private final BiPredicate<PropertyDescriptor, Class<?>> acceptor;

    BeanPropertyType(String name, Class<?>... classes) {
        this.name = name;
        this.classes = Arrays.asList(classes);
        this.acceptor = null;
    }

    private final String name;

    private final List<Class<?>> classes;

    BeanPropertyType(String name, BiPredicate<PropertyDescriptor, Class<?>> predicate) {
        this.name = name;
        this.classes = null;
        this.acceptor = predicate;
    }

    public static BeanPropertyType of(PropertyDescriptor descriptor, Class<?> clazz) {
        for (BeanPropertyType type : values()) {
            if (null != type.classes && type.classes.stream().anyMatch(item -> item.isAssignableFrom(descriptor.getPropertyType())) ||
                    null != type.acceptor && type.acceptor.test(descriptor, clazz)) {
                return type;
            }
        }
        return null;
    }
}
