package com.flyfish.framework.beans.resolver;

import com.flyfish.framework.compiler.DynamicJavaCompiler;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.annotation.Lazy;
import org.springframework.web.reactive.result.method.AbstractHandlerMethodMapping;
import org.springframework.web.reactive.result.method.annotation.RequestMappingHandlerMapping;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 动态Bean生成解析器
 *
 * @author wangyu
 */
@Slf4j
@Lazy(false)
public class DynamicRestBeanResolver implements InitializingBean, DisposableBean {

    private static final List<String> controllers = new ArrayList<>();
    private static final Set<Class<?>> classes = new HashSet<>();
    private final Method handlerRegister;
    private final RequestMappingHandlerMapping mapping;

    public DynamicRestBeanResolver(RequestMappingHandlerMapping mapping) throws NoSuchMethodException {
        this.mapping = mapping;
        this.handlerRegister = AbstractHandlerMethodMapping.class.getDeclaredMethod("detectHandlerMethods", Object.class);
    }

    public static void addController(String controller) {
        controllers.add(controller);
    }

    public static void setClasses(Set<Class<?>> classSet) {
        classes.addAll(classSet);
    }

    public Set<Class<?>> getClasses() {
        return classes;
    }

    @Override
    public void destroy() throws Exception {
        log.info("正在销毁rest自动配置器");
        DynamicJavaCompiler.delegate().close();
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        log.info("正在注册controller...");
        for (String bean : controllers) {
            log.info("注册controller：{}", bean);
            handlerRegister.setAccessible(true);
            handlerRegister.invoke(mapping, bean);
        }
    }
}
